@@ -1,7 +1,7 @@
 /* imagsize:  figure out the image size of BMP, GIF, JPEG, PBM, PGM, PPM, PNG,
               TIFF, XBM, and XPM files
 
-   Original "image_size" program Copyright (C)1997,1998 by Jef Poskanzer
+   Original "image_size" program Copyright (C)1997,1998,2014 by Jef Poskanzer
    <jef@acme.com>.  All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
@@ -41,8 +41,8 @@
 
 #include "../config.h"
 #include <limits.h>
-#include <stdio.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
 
@@ -180,7 +180,7 @@ get (FILE *f, unsigned char *chP)
 {
   int ich = getc (f);
   if (ich == EOF)
-    return (failure ("unknown-EOF"));
+    return (failure ("imagsize error: unknown-EOF"));
   *chP = (unsigned char) ich;
   return (OK);
 }
@@ -196,7 +196,7 @@ mustbe (FILE *f, unsigned char ch)
     return (NOT_OK);
   if (ch2 != ch)
     {
-      sprintf (buffer, "corrupted image -- expecting %x at %ld",
+      sprintf (buffer, "imagsize error: corrupted image -- expecting %x at %ld",
         (unsigned int) ch, ftell (f));
       return (failure (buffer));
     }
@@ -214,7 +214,7 @@ gif (FILE *f, int *widthP, int *heightP, int *depth)
   if (!mustbe (f, (unsigned char) '8')) return (NOT_OK);
   if (!get (f, &ch)) return (NOT_OK);
   if (ch != (unsigned char) '7' && ch != (unsigned char) '9')
-    return (failure ("corrupted image file"));
+    return (failure ("imagsize error: corrupted image file"));
   if (!mustbe (f, 'a')) return (NOT_OK);
 
   /* Width and height are the next things in the file. */
@@ -431,7 +431,7 @@ jpeg (FILE *f, int *widthP, int *heightP, double *qual,
           break;
         }
     }
-  return (failure ("JPEG size not found"));
+  return (failure ("imagsize error: JPEG size not found"));
 }
 
 
@@ -441,13 +441,13 @@ xbm (FILE *f, int *widthP, int *heightP)
   char line[500], name_and_type[500];
 
   if (fgets (line, (int) sizeof (line), f) == NULL)
-    return (failure ("XBM width not found"));
+    return (failure ("imagsize error: XBM width not found"));
   if (sscanf (line, "efine %s %d", name_and_type, widthP) != 2)
-    return (failure ("XBM bogus width"));
+    return (failure ("imagsize error: XBM bogus width"));
   if (fgets (line, (int) sizeof (line), f) == NULL)
-    return (failure ("XBM height not found"));
+    return (failure ("Ximagsize error: BM height not found"));
   if (sscanf (line, "#define %s %d", name_and_type, heightP) != 2)
-    return (failure ("XBM bogus height"));
+    return (failure ("imagsize error: XBM bogus height"));
 
   /* indicate successful finish */
   return (OK);
@@ -488,7 +488,7 @@ xpm (FILE *f, int *widthP, int *heightP, int *depth)
   /* complain if not found */
   if (done == 0) {
     *widthP = *heightP = *depth = 0;
-    return (failure ("XPM width and height not found"));
+    return (failure ("imagsize error: XPM width and height not found"));
   }
 
   /* indicate successful finish */
@@ -511,7 +511,7 @@ ppm (FILE *f, int *widthP, int *heightP)
   /* complain if not found */
   if (done == 0) {
     *widthP = *heightP = 0;
-    return (failure ("PPM width and height not found"));
+    return (failure ("imagsize error: PPM width and height not found"));
   }
 
   /* indicate successful finish */
@@ -761,7 +761,7 @@ png (FILE *f, int *widthP, int *heightP, int *bpp, int *typenum)
       /* Not the block we want.  Skip it. */
       if (fseek (f, l + 4, 1)) return (NOT_OK);
     }
-  return (failure ("PNG size not found"));
+  return (failure ("imagsize error: PNG size not found"));
 }
 
 
@@ -890,7 +890,7 @@ image_size (FILE *f, int *widthP, int *heightP, double *qual,
   else
     {
       strcpy (type, "unknown");
-      return (failure ("unknown"));
+      return (failure ("imagsize warning: unknown image type"));
     }
 }
 
@@ -958,7 +958,7 @@ print_size (char *filename, int format, int verbose)
 
 
 int
-main (int argc, char *argv[])
+main (int argc, char **argv)
 {
   FILE *infile;
   char filename[PATH_MAX], *listname, *newline, *rc;
