@@ -1,6 +1,6 @@
 /* filehash:  Print various hash digests and filesizes for specified files.
 
-   Copyright (C) 2004-2011 by Brian Lindholm.
+   Copyright (C) 2004-2018 by Brian Lindholm.
    This file is part of the littleutils utility set.
 
    The filehash utility is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
    more details.
 
    You should have received a copy of the GNU General Public License along with
-   the littleutils.  If not, see <http://www.gnu.org/licenses/>. */
+   the littleutils.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #include "../config.h"
@@ -84,7 +84,7 @@ help (FILE *where)
 static void
 print_filehash (char *filename, off_t read_bytes, int print_size, int run_md5,
   int run_sha1, int run_sha224, int run_sha256, int run_sha384, int run_sha512,
-  int verbose)
+  int verbose, int classic)
 {
   FILE *file;
   int i, not_first, rc;
@@ -158,58 +158,107 @@ print_filehash (char *filename, off_t read_bytes, int print_size, int run_md5,
           }
           (void) fclose (file);
           not_first = 0;
-          if (verbose == 1) {
+          if ((verbose == 1) && (classic == 0)) {
             fprintf (stdout, "%s", filename);
             not_first = 1;
           }
           if (print_size) {
-            if (not_first)
-              fprintf (stdout, "\t");
-            fprintf (stdout, sizefmt, file_stats.st_size);
-            not_first = 1;
+            if (classic) {
+              fprintf (stdout, sizefmt, file_stats.st_size);
+              fprintf (stdout, "  %s\n", filename);
+            }
+            else {
+              if (not_first)
+                fprintf (stdout, "\t");
+              fprintf (stdout, sizefmt, file_stats.st_size);
+              not_first = 1;
+            }
           }
           if (run_md5) {
-            if (not_first)
-              fprintf (stdout, "\t");
-            for (i = 0; i < 16; ++i)
-              fprintf (stdout, "%02x", (unsigned int) md5result[i]);
-            not_first = 1;
+            if (classic) {
+              for (i = 0; i < 16; ++i)
+                fprintf (stdout, "%02x", (unsigned int) md5result[i]);
+              fprintf (stdout, "  %s\n", filename);
+            }
+            else {
+              if (not_first)
+                fprintf (stdout, "\t");
+              for (i = 0; i < 16; ++i)
+                fprintf (stdout, "%02x", (unsigned int) md5result[i]);
+              not_first = 1;
+            }
           }
           if (run_sha1) {
-            if (not_first)
-              fprintf (stdout, "\t");
-            for (i = 0; i < 20; ++i)
-              fprintf (stdout, "%02x", (unsigned int) sha1result[i]);
-            not_first = 1;
+            if (classic) {
+              for (i = 0; i < 20; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha1result[i]);
+              fprintf (stdout, "  %s\n", filename);
+            }
+            else {
+              if (not_first)
+                fprintf (stdout, "\t");
+              for (i = 0; i < 20; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha1result[i]);
+              not_first = 1;
+            }
           }
           if (run_sha224) {
-            if (not_first)
-              fprintf (stdout, "\t");
-            for (i = 0; i < 28; ++i)
-              fprintf (stdout, "%02x", (unsigned int) sha224result[i]);
-            not_first = 1;
+            if (classic) {
+              for (i = 0; i < 28; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha224result[i]);
+              fprintf (stdout, "  %s\n", filename);
+            }
+            else {
+              if (not_first)
+                fprintf (stdout, "\t");
+              for (i = 0; i < 28; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha224result[i]);
+              not_first = 1;
+            }
           }
           if (run_sha256) {
-            if (not_first)
-              fprintf (stdout, "\t");
-            for (i = 0; i < 32; ++i)
-              fprintf (stdout, "%02x", (unsigned int) sha256result[i]);
-            not_first = 1;
+            if (classic) {
+              for (i = 0; i < 32; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha256result[i]);
+              fprintf (stdout, "  %s\n", filename);
+            }
+            else {
+              if (not_first)
+                fprintf (stdout, "\t");
+              for (i = 0; i < 32; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha256result[i]);
+              not_first = 1;
+            }
           }
           if (run_sha384) {
-            if (not_first)
-              fprintf (stdout, "\t");
-            for (i = 0; i < 48; ++i)
-              fprintf (stdout, "%02x", (unsigned int) sha384result[i]);
-            not_first = 1;
+            if (classic) {
+              for (i = 0; i < 48; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha384result[i]);
+              fprintf (stdout, "  %s\n", filename);
+            }
+            else {
+              if (not_first)
+                fprintf (stdout, "\t");
+              for (i = 0; i < 48; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha384result[i]);
+              not_first = 1;
+            }
           }
           if (run_sha512) {
-            if (not_first)
-              fprintf (stdout, "\t");
-            for (i = 0; i < 64; ++i)
-              fprintf (stdout, "%02x", (unsigned int) sha512result[i]);
+            if (classic) {
+              for (i = 0; i < 64; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha512result[i]);
+              fprintf (stdout, "  %s\n", filename);
+            }
+            else {
+              if (not_first)
+                fprintf (stdout, "\t");
+              for (i = 0; i < 64; ++i)
+                fprintf (stdout, "%02x", (unsigned int) sha512result[i]);
+            }
           }
-          fprintf (stdout, "\n");
+          if (classic == 0)
+            fprintf (stdout, "\n");
         }
     }
 }
@@ -220,12 +269,13 @@ main (int argc, char **argv)
 {
   FILE *infile;
   char filename[PATH_MAX], *listname, *newline, *rc;
-  int argn, opt, print_size, run_md5, run_sha1, run_sha224, run_sha256,
-      run_sha384, run_sha512, use_file, use_pipe, verbose;
+  int argn, classic, opt, print_size, run_md5, run_sha1, run_sha224,
+      run_sha256, run_sha384, run_sha512, use_file, use_pipe, verbose;
   off_t read_bytes;
 
   /* parse options */
 
+  classic = 0;
   listname = "";
   print_size = 0;
   read_bytes = 0;
@@ -238,7 +288,7 @@ main (int argc, char **argv)
   use_file = 0;
   use_pipe = 0;
   verbose = 0;
-  while ((opt = getopt (argc, argv, "123456f:hpn:qsv")) != OPTEND)
+  while ((opt = getopt (argc, argv, "123456cf:hpn:qsv")) != OPTEND)
     switch (opt)
       {
       case '1':
@@ -259,6 +309,9 @@ main (int argc, char **argv)
       case '6':
         run_sha512 = 1;
         break;
+      case 'c':
+        classic = 1;
+        break;
       case 'f':
         use_file = 1;
         listname = optarg;
@@ -325,7 +378,7 @@ main (int argc, char **argv)
                   if (strlen (filename) != 0)
                     print_filehash (filename, read_bytes, print_size, run_md5,
                       run_sha1, run_sha224, run_sha256, run_sha384, run_sha512,
-                      verbose);
+                      verbose, classic);
                 }
             }
           (void) fclose (infile);
@@ -346,7 +399,7 @@ main (int argc, char **argv)
             if (strlen (filename) != 0)
               print_filehash (filename, read_bytes, print_size, run_md5,
                 run_sha1, run_sha224, run_sha256, run_sha384, run_sha512,
-                verbose);
+                verbose, classic);
           }
       }
 
@@ -354,7 +407,7 @@ main (int argc, char **argv)
 
   for (argn = optind; argn < argc; argn++)
     print_filehash (argv[argn], read_bytes, print_size, run_md5, run_sha1,
-      run_sha224, run_sha256, run_sha384, run_sha512, verbose);
+      run_sha224, run_sha256, run_sha384, run_sha512, verbose, classic);
 
   /* indicate successful finish */
 
