@@ -1,6 +1,6 @@
 /* randomize:  Randomizes lines from a file or from stdin.
 
-   Copyright (C) 2004-2014 by Brian Lindholm.
+   Copyright (C) 2004-2018 by Brian Lindholm.
    This file is part of the littleutils utility set.
 
    The randomize utility is free software; you can redistribute it and/or
@@ -14,10 +14,11 @@
    more details.
 
    You should have received a copy of the GNU General Public License along with
-   the littleutils.  If not, see <http://www.gnu.org/licenses/>. */
+   the littleutils.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #include "../config.h"
+#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -71,23 +72,38 @@ help (FILE *where)
 static void
 rand_seed ()
 {
+  int URANDOM, urandom_worked = 1;
+  ssize_t bytes;
   time_t junk;
   unsigned short seed[3];
 
-  /* initialize random number generator */
-  (void) time (&junk);
+  /* pre-populate arracy for seed using /dev/urandom */
+  URANDOM = open("/dev/urandom", O_RDONLY);
+  if (URANDOM < 0) {
+    urandom_worked = 0;
+  }
+  else {
+    bytes = read(URANDOM, seed, sizeof seed);
+    if (bytes < 6) {
+      urandom_worked = 0;
+    }
+  }
+  close(URANDOM);
+
+  /* fall back to time and PID combo if /dev/urandom failed */
+  if (urandom_worked == 0) {
+    (void) time (&junk);
+    seed[0] = (unsigned short) (junk & 0xFFFF);
+    seed[1] = (unsigned short) (getpid () & 0xFFFF);
+    seed[2] = (unsigned short) ((junk >> 16) & 0xFFFF);
+  }
+
+  /* initialize random number generator seed */
 #ifdef HAVE_LRAND48
-  seed[0] = (unsigned short) (junk & 0xFFFF);
-  seed[1] = (unsigned short) ((junk >> 16) & 0xFFFF);
-  seed[2] = (unsigned short) (getpid () & 0xFFFF);
   (void) seed48 (seed);
 #elif defined(HAVE_RANDOM)
-  seed[0] = (unsigned short) (junk & 0xFFFF);
-  seed[1] = (unsigned short) (getpid () & 0xFFFF);
   srandom (((unsigned int) seed[0] << 16) + (unsigned int) seed[1]);
 #else
-  seed[0] = (unsigned short) (junk & 0xFFFF);
-  seed[1] = (unsigned short) (getpid () & 0xFFFF);
   srand (((unsigned int) seed[0] << 16) + (unsigned int) seed[1]);
 #endif
 }
