@@ -1,7 +1,7 @@
 /* tempname:  Creates a unique temporary file (or filename) for use by shell
    scripts.
 
-   Copyright (C) 2004-2013 by Brian Lindholm.
+   Copyright (C) 2004-2018 by Brian Lindholm.
    This file is part of the littleutils utility set.
 
    The tempname utility is free software; you can redistribute it and/or
@@ -15,7 +15,7 @@
    more details.
 
    You should have received a copy of the GNU General Public License along with
-   the littleutils.  If not, see <http://www.gnu.org/licenses/>. */
+   the littleutils.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #include "../config.h"
@@ -128,15 +128,57 @@ good_dir (const char *path, int verbose)
 }
 
 
+/* random seed function */
+
+static void
+rand_seed ()
+{
+  int URANDOM, urandom_worked = 1;
+  ssize_t bytes;
+  time_t junk;
+  unsigned short seed[3];
+
+  /* pre-populate arracy for seed using /dev/urandom */
+  URANDOM = open("/dev/urandom", O_RDONLY);
+  if (URANDOM < 0) {
+    urandom_worked = 0;
+  }
+  else {
+    bytes = read(URANDOM, seed, sizeof seed);
+    if (bytes < 6) {
+      urandom_worked = 0;
+    }
+  }
+  close(URANDOM);
+
+  /* fall back to time and PID combo if /dev/urandom failed */
+  if (urandom_worked == 0) {
+    (void) time (&junk);
+    seed[0] = (unsigned short) (junk & 0xFFFF);
+    seed[1] = (unsigned short) (getpid () & 0xFFFF);
+    seed[2] = (unsigned short) ((junk >> 16) & 0xFFFF);
+  }
+
+  /* initialize random number generator seed */
+#ifdef HAVE_LRAND48
+  (void) seed48 (seed);
+#elif defined(HAVE_RANDOM)
+  srandom (((unsigned int) seed[0] << 16) + (unsigned int) seed[1]);
+#else
+  srand (((unsigned int) seed[0] << 16) + (unsigned int) seed[1]);
+#endif
+}
+
+
 /* special variant of mkstemp, allows X's to occur in non-end locations */
 
 static int
 mkstemp_custom (char *tmpl, int kind)
 {
   char *copy;
-  int count, fd, i, len, rc;
+  int count, fd, rc;
   int save_errno = errno;
-  time_t junk;
+  size_t i, len;
   struct stat file_stats;
 
   /* characters used in temporary file names */
@@ -144,19 +186,13 @@ mkstemp_custom (char *tmpl, int kind)
   "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
 
   /* determine length of template and allocate storage */
-  len = (int) strlen (tmpl);
+  len = strlen (tmpl);
   copy = (char *) malloc ((len + 1) * sizeof (char));
 
   /* initialize random number generator */
-  time (&junk);
-#ifdef HAVE_LRAND48
-  srand48 (((long) junk + (long) getpid ()) % (long) 1073741824);
-#elif defined(HAVE_RANDOM)
-  srandom (((unsigned int) junk + (unsigned int) getpid ()) % (unsigned int) 1073741824);
-#else
-  srand (((unsigned int) junk + (unsigned int) getpid ()) % (unsigned int) 1073741824);
-#endif
+  rand_seed ();
 
+  /* generate the random name */
   for (count = 0; count < TMP_MAX; ++count)
     {
       strcpy (copy, tmpl);
